@@ -16,16 +16,17 @@ def canonicalize(expression: exp.Expression) -> exp.Expression:
     Args:
         expression: The expression to canonicalize.
     """
-    exp.replace_children(expression, canonicalize)
 
-    expression = add_text_to_concat(expression)
-    expression = replace_date_funcs(expression)
-    expression = coerce_type(expression)
-    expression = remove_redundant_casts(expression)
-    expression = ensure_bools(expression, _replace_int_predicate)
-    expression = remove_ascending_order(expression)
+    def _canonicalize(expression: exp.Expression) -> exp.Expression:
+        expression = add_text_to_concat(expression)
+        expression = replace_date_funcs(expression)
+        expression = coerce_type(expression)
+        expression = remove_redundant_casts(expression)
+        expression = ensure_bools(expression, _replace_int_predicate)
+        expression = remove_ascending_order(expression)
+        return expression
 
-    return expression
+    return exp.replace_tree(expression, _canonicalize)
 
 
 def add_text_to_concat(node: exp.Expression) -> exp.Expression:
@@ -38,7 +39,12 @@ def replace_date_funcs(node: exp.Expression) -> exp.Expression:
     if isinstance(node, exp.Date) and not node.expressions and not node.args.get("zone"):
         return exp.cast(node.this, to=exp.DataType.Type.DATE)
     if isinstance(node, exp.Timestamp) and not node.expression:
-        return exp.cast(node.this, to=exp.DataType.Type.TIMESTAMP)
+        if not node.type:
+            from sqlglot.optimizer.annotate_types import annotate_types
+
+            node = annotate_types(node)
+        return exp.cast(node.this, to=node.type or exp.DataType.Type.TIMESTAMP)
+
     return node
 
 
@@ -76,9 +82,8 @@ def coerce_type(node: exp.Expression) -> exp.Expression:
 def remove_redundant_casts(expression: exp.Expression) -> exp.Expression:
     if (
         isinstance(expression, exp.Cast)
-        and expression.to.type
         and expression.this.type
-        and expression.to.type.this == expression.this.type.this
+        and expression.to.this == expression.this.type.this
     ):
         return expression.this
     return expression
@@ -165,7 +170,7 @@ def _replace_cast(node: exp.Expression, to: exp.DataType.Type) -> None:
 # with y as (select true as x) select x = 0 FROM y -- illegal presto query
 def _replace_int_predicate(expression: exp.Expression) -> None:
     if isinstance(expression, exp.Coalesce):
-        for _, child in expression.iter_expressions():
+        for child in expression.iter_expressions():
             _replace_int_predicate(child)
     elif expression.type and expression.type.this in exp.DataType.INTEGER_TYPES:
         expression.replace(expression.neq(0))
