@@ -1,12 +1,12 @@
 from __future__ import annotations
 
 import logging
-import typing as t
 
 from sqlglot import exp
 from sqlglot.errors import OptimizeError
 from sqlglot.helper import while_changing
-from sqlglot.optimizer.simplify import flatten, uniq_sort
+from sqlglot.optimizer.scope import find_all_in_scope
+from sqlglot.optimizer.simplify import flatten, rewrite_between, uniq_sort
 
 logger = logging.getLogger("sqlglot")
 
@@ -28,13 +28,14 @@ def normalize(expression: exp.Expression, dnf: bool = False, max_distance: int =
     Returns:
         sqlglot.Expression: normalized expression
     """
-    cache: t.Dict[int, str] = {}
-
-    for node, *_ in tuple(expression.walk(prune=lambda e, *_: isinstance(e, exp.Connector))):
+    for node in tuple(expression.walk(prune=lambda e: isinstance(e, exp.Connector))):
         if isinstance(node, exp.Connector):
             if normalized(node, dnf=dnf):
                 continue
+            root = node is expression
+            original = node.copy()
 
+            node.transform(rewrite_between, copy=False)
             distance = normalization_distance(node, dnf=dnf)
 
             if distance > max_distance:
@@ -43,11 +44,9 @@ def normalize(expression: exp.Expression, dnf: bool = False, max_distance: int =
                 )
                 return expression
 
-            root = node is expression
-            original = node.copy()
             try:
                 node = node.replace(
-                    while_changing(node, lambda e: distributive_law(e, dnf, max_distance, cache))
+                    while_changing(node, lambda e: distributive_law(e, dnf, max_distance))
                 )
             except OptimizeError as e:
                 logger.info(e)
@@ -62,15 +61,33 @@ def normalize(expression: exp.Expression, dnf: bool = False, max_distance: int =
     return expression
 
 
-def normalized(expression, dnf=False):
-    ancestor, root = (exp.And, exp.Or) if dnf else (exp.Or, exp.And)
-
-    return not any(connector.find_ancestor(ancestor) for connector in expression.find_all(root))
-
-
-def normalization_distance(expression, dnf=False):
+def normalized(expression: exp.Expression, dnf: bool = False) -> bool:
     """
-    The difference in the number of predicates between the current expression and the normalized form.
+    Checks whether a given expression is in a normal form of interest.
+
+    Example:
+        >>> from sqlglot import parse_one
+        >>> normalized(parse_one("(a AND b) OR c OR (d AND e)"), dnf=True)
+        True
+        >>> normalized(parse_one("(a OR b) AND c"))  # Checks CNF by default
+        True
+        >>> normalized(parse_one("a AND (b OR c)"), dnf=True)
+        False
+
+    Args:
+        expression: The expression to check if it's normalized.
+        dnf: Whether to check if the expression is in Disjunctive Normal Form (DNF).
+            Default: False, i.e. we check if it's in Conjunctive Normal Form (CNF).
+    """
+    ancestor, root = (exp.And, exp.Or) if dnf else (exp.Or, exp.And)
+    return not any(
+        connector.find_ancestor(ancestor) for connector in find_all_in_scope(expression, root)
+    )
+
+
+def normalization_distance(expression: exp.Expression, dnf: bool = False) -> int:
+    """
+    The difference in the number of predicates between a given expression and its normalized form.
 
     This is used as an estimate of the cost of the conversion which is exponential in complexity.
 
@@ -81,10 +98,12 @@ def normalization_distance(expression, dnf=False):
         4
 
     Args:
-        expression (sqlglot.Expression): expression to compute distance
-        dnf (bool): compute to dnf distance instead
+        expression: The expression to compute the normalization distance for.
+        dnf: Whether to check if the expression is in Disjunctive Normal Form (DNF).
+            Default: False, i.e. we check if it's in Conjunctive Normal Form (CNF).
+
     Returns:
-        int: difference
+        The normalization distance.
     """
     return sum(_predicate_lengths(expression, dnf)) - (
         sum(1 for _ in expression.find_all(exp.Connector)) + 1
@@ -111,7 +130,7 @@ def _predicate_lengths(expression, dnf):
     return _predicate_lengths(left, dnf) + _predicate_lengths(right, dnf)
 
 
-def distributive_law(expression, dnf, max_distance, cache=None):
+def distributive_law(expression, dnf, max_distance):
     """
     x OR (y AND z) -> (x OR y) AND (x OR z)
     (x AND y) OR (y AND z) -> (x OR y) AND (x OR z) AND (y OR y) AND (y OR z)
@@ -124,7 +143,7 @@ def distributive_law(expression, dnf, max_distance, cache=None):
     if distance > max_distance:
         raise OptimizeError(f"Normalization distance {distance} exceeds max {max_distance}")
 
-    exp.replace_children(expression, lambda e: distributive_law(e, dnf, max_distance, cache))
+    exp.replace_children(expression, lambda e: distributive_law(e, dnf, max_distance))
     to_exp, from_exp = (exp.Or, exp.And) if dnf else (exp.And, exp.Or)
 
     if isinstance(expression, from_exp):
@@ -135,30 +154,30 @@ def distributive_law(expression, dnf, max_distance, cache=None):
 
         if isinstance(a, to_exp) and isinstance(b, to_exp):
             if len(tuple(a.find_all(exp.Connector))) > len(tuple(b.find_all(exp.Connector))):
-                return _distribute(a, b, from_func, to_func, cache)
-            return _distribute(b, a, from_func, to_func, cache)
+                return _distribute(a, b, from_func, to_func)
+            return _distribute(b, a, from_func, to_func)
         if isinstance(a, to_exp):
-            return _distribute(b, a, from_func, to_func, cache)
+            return _distribute(b, a, from_func, to_func)
         if isinstance(b, to_exp):
-            return _distribute(a, b, from_func, to_func, cache)
+            return _distribute(a, b, from_func, to_func)
 
     return expression
 
 
-def _distribute(a, b, from_func, to_func, cache):
+def _distribute(a, b, from_func, to_func):
     if isinstance(a, exp.Connector):
         exp.replace_children(
             a,
             lambda c: to_func(
-                uniq_sort(flatten(from_func(c, b.left)), cache),
-                uniq_sort(flatten(from_func(c, b.right)), cache),
+                uniq_sort(flatten(from_func(c, b.left))),
+                uniq_sort(flatten(from_func(c, b.right))),
                 copy=False,
             ),
         )
     else:
         a = to_func(
-            uniq_sort(flatten(from_func(a, b.left)), cache),
-            uniq_sort(flatten(from_func(a, b.right)), cache),
+            uniq_sort(flatten(from_func(a, b.left))),
+            uniq_sort(flatten(from_func(a, b.right))),
             copy=False,
         )
 
