@@ -120,6 +120,8 @@ def _pop_table_column_aliases(derived_tables: t.List[exp.CTE | exp.Subquery]) ->
     For example, `col1` and `col2` will be dropped in SELECT ... FROM (SELECT ...) AS foo(col1, col2)
     """
     for derived_table in derived_tables:
+        if isinstance(derived_table.parent, exp.With) and derived_table.parent.recursive:
+            continue
         table_alias = derived_table.args.get("alias")
         if table_alias:
             table_alias.args.pop("columns", None)
@@ -207,14 +209,20 @@ def _expand_alias_refs(scope: Scope, resolver: Resolver) -> None:
         if not node:
             return
 
-        for column, *_ in walk_in_scope(node, prune=lambda node, *_: node.is_star):
+        for column in walk_in_scope(node, prune=lambda node: node.is_star):
             if not isinstance(column, exp.Column):
                 continue
 
             table = resolver.get_table(column.name) if resolve_table and not column.table else None
             alias_expr, i = alias_to_expression.get(column.name, (None, 1))
             double_agg = (
-                (alias_expr.find(exp.AggFunc) and column.find_ancestor(exp.AggFunc))
+                (
+                    alias_expr.find(exp.AggFunc)
+                    and (
+                        column.find_ancestor(exp.AggFunc)
+                        and not isinstance(column.find_ancestor(exp.Window, exp.Select), exp.Window)
+                    )
+                )
                 if alias_expr
                 else False
             )
@@ -298,7 +306,7 @@ def _expand_positional_references(
             else:
                 select = select.this
 
-                if isinstance(select, exp.Literal):
+                if isinstance(select, exp.CONSTANTS) or select.find(exp.Explode, exp.Unnest):
                     new_nodes.append(node)
                 else:
                     new_nodes.append(select.copy())
@@ -404,7 +412,7 @@ def _expand_stars(
             tables = list(scope.selected_sources)
             _add_except_columns(expression, tables, except_columns)
             _add_replace_columns(expression, tables, replace_columns)
-        elif expression.is_star:
+        elif expression.is_star and not isinstance(expression, exp.Dot):
             tables = [expression.table]
             _add_except_columns(expression.this, tables, except_columns)
             _add_replace_columns(expression.this, tables, replace_columns)
@@ -417,7 +425,7 @@ def _expand_stars(
                 raise OptimizeError(f"Unknown table: {table}")
 
             columns = resolver.get_source_columns(table, only_visible=True)
-            columns = columns or scope.outer_column_list
+            columns = columns or scope.outer_columns
 
             if pseudocolumns:
                 columns = [name for name in columns if name.upper() not in pseudocolumns]
@@ -437,7 +445,7 @@ def _expand_stars(
 
                 if pivot_columns:
                     new_selections.extend(
-                        exp.alias_(exp.column(name, table=pivot.alias), name, copy=False)
+                        alias(exp.column(name, table=pivot.alias), name, copy=False)
                         for name in pivot_columns
                         if name not in columns_to_exclude
                     )
@@ -509,7 +517,7 @@ def qualify_outputs(scope_or_expression: Scope | exp.Expression) -> None:
 
     new_selections = []
     for i, (selection, aliased_column) in enumerate(
-        itertools.zip_longest(scope.expression.selects, scope.outer_column_list)
+        itertools.zip_longest(scope.expression.selects, scope.outer_columns)
     ):
         if selection is None:
             break
@@ -536,7 +544,7 @@ def quote_identifiers(expression: E, dialect: DialectType = None, identify: bool
     """Makes sure all identifiers that need to be quoted are quoted."""
     return expression.transform(
         Dialect.get_or_raise(dialect).quote_identifier, identify=identify, copy=False
-    )
+    )  # type: ignore
 
 
 def pushdown_cte_alias_columns(expression: exp.Expression) -> exp.Expression:
